@@ -44,14 +44,17 @@ pub struct State<'a> {
     pub table_alignments: Vec<Alignment>,
     /// Keeps the current table headers, if we are currently serializing a table.
     pub table_headers: Vec<String>,
-    /// If set, the next 'text' will be stored for later use
-    pub store_next_text: bool,
     /// The last seen text when serializing a header
     pub text_for_header: Option<String>,
     /// Is set while we are handling text in a code block
     pub is_in_code_block: bool,
     /// True if the last event was html. Used to inject additional newlines to support markdown inside of HTML tags.
     pub last_was_html: bool,
+
+    /// Keeps track of the last seen shortcut/link
+    pub current_shortcut_text: Option<String>,
+    /// A list of shortcuts seen so far for later emission
+    pub shortcuts: Vec<(String, String, String)>,
 }
 
 /// Configuration for the [`cmark()`] function.
@@ -147,9 +150,6 @@ where
     F: fmt::Write,
 {
     let mut state = state.unwrap_or_default();
-    let mut current_shortcut = String::new();
-    let mut inside_shortcut = false;
-    let mut shortcuts: Vec<(String, String, String)> = Vec::new();
     fn padding<'a, F>(f: &mut F, p: &[Cow<'a, str>]) -> fmt::Result
     where
         F: fmt::Write,
@@ -275,10 +275,9 @@ where
                 formatter.write_str("---")
             }
             Code(ref text) => {
-                if state.store_next_text {
-                    state.store_next_text = false;
+                if let Some(text_for_header) = state.text_for_header.as_mut() {
                     let code = format!("{}{}{}", options.code_block_token, text, options.code_block_token);
-                    state.text_for_header = Some(code)
+                    text_for_header.push_str(&code);
                 }
                 formatter
                     .write_char(options.code_block_token)
@@ -312,12 +311,12 @@ where
                     TableHead => Ok(()),
                     TableRow => Ok(()),
                     TableCell => {
-                        state.store_next_text = true;
+                        state.text_for_header = Some(String::new());
                         formatter.write_char('|')
                     }
                     Link(LinkType::Autolink | LinkType::Email, ..) => formatter.write_char('<'),
                     Link(LinkType::Shortcut, ..) => {
-                        inside_shortcut = true;
+                        state.current_shortcut_text = Some(String::new());
                         formatter.write_char('[')
                     }
                     Link(..) => formatter.write_char('['),
@@ -384,10 +383,10 @@ where
             End(ref tag) => match tag {
                 Link(LinkType::Autolink | LinkType::Email, ..) => formatter.write_char('>'),
                 Link(LinkType::Shortcut, ref uri, ref title) => {
-                    if inside_shortcut {
-                        shortcuts.push((current_shortcut.clone(), uri.to_string(), title.to_string()));
-                        inside_shortcut = false;
-                        current_shortcut = String::new();
+                    if let Some(shortcut_text) = state.current_shortcut_text.take() {
+                        state
+                            .shortcuts
+                            .push((shortcut_text, uri.to_string(), title.to_string()));
                     }
                     formatter.write_char(']')
                 }
@@ -448,10 +447,13 @@ where
                     Ok(())
                 }
                 TableCell => {
-                    state.table_headers.push(match state.text_for_header.take() {
-                        Some(text) => text,
-                        None => "  ".into(),
-                    });
+                    state.table_headers.push(
+                        state
+                            .text_for_header
+                            .take()
+                            .filter(|s| !s.is_empty())
+                            .unwrap_or_else(|| "  ".into()),
+                    );
                     Ok(())
                 }
                 ref t @ TableRow | ref t @ TableHead => {
@@ -515,12 +517,11 @@ where
             HardBreak => formatter.write_str("  \n").and(padding(&mut formatter, &state.padding)),
             SoftBreak => formatter.write_char('\n').and(padding(&mut formatter, &state.padding)),
             Text(ref text) => {
-                if inside_shortcut {
-                    current_shortcut.push_str(text);
+                if let Some(shortcut_text) = state.current_shortcut_text.as_mut() {
+                    shortcut_text.push_str(text);
                 }
-                if state.store_next_text {
-                    state.store_next_text = false;
-                    state.text_for_header = Some(text.to_owned().into_string())
+                if let Some(text_for_header) = state.text_for_header.as_mut() {
+                    text_for_header.push_str(text)
                 }
                 consume_newlines(&mut formatter, &mut state)?;
                 print_text_without_trailing_newline(
@@ -541,9 +542,9 @@ where
             }
         }?
     }
-    if !shortcuts.is_empty() {
+    if !state.shortcuts.is_empty() {
         formatter.write_str("\n")?;
-        for shortcut in shortcuts {
+        for shortcut in state.shortcuts.drain(..) {
             write!(formatter, "\n[{}", shortcut.0)?;
             close_link(&shortcut.1, &shortcut.2, &mut formatter, LinkType::Shortcut)?
         }
